@@ -248,34 +248,42 @@ extern int yydebug;
     VAR_USE_SYSTEMD = 458,
     VAR_SHM_ENABLE = 459,
     VAR_SHM_KEY = 460,
-    VAR_DNSCRYPT = 461,
-    VAR_DNSCRYPT_ENABLE = 462,
-    VAR_DNSCRYPT_PORT = 463,
-    VAR_DNSCRYPT_PROVIDER = 464,
-    VAR_DNSCRYPT_SECRET_KEY = 465,
-    VAR_DNSCRYPT_PROVIDER_CERT = 466,
-    VAR_DNSCRYPT_PROVIDER_CERT_ROTATED = 467,
-    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE = 468,
-    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS = 469,
-    VAR_DNSCRYPT_NONCE_CACHE_SIZE = 470,
-    VAR_DNSCRYPT_NONCE_CACHE_SLABS = 471,
-    VAR_IPSECMOD_ENABLED = 472,
-    VAR_IPSECMOD_HOOK = 473,
-    VAR_IPSECMOD_IGNORE_BOGUS = 474,
-    VAR_IPSECMOD_MAX_TTL = 475,
-    VAR_IPSECMOD_WHITELIST = 476,
-    VAR_IPSECMOD_STRICT = 477,
-    VAR_CACHEDB = 478,
-    VAR_CACHEDB_BACKEND = 479,
-    VAR_CACHEDB_SECRETSEED = 480,
-    VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM = 481,
-    VAR_FOR_UPSTREAM = 482,
-    VAR_AUTH_ZONE = 483,
-    VAR_ZONEFILE = 484,
-    VAR_MASTER = 485,
-    VAR_URL = 486,
-    VAR_FOR_DOWNSTREAM = 487,
-    VAR_FALLBACK_ENABLED = 488
+    VAR_ROOT_KEY_SENTINEL = 461,
+    VAR_DNSCRYPT = 462,
+    VAR_DNSCRYPT_ENABLE = 463,
+    VAR_DNSCRYPT_PORT = 464,
+    VAR_DNSCRYPT_PROVIDER = 465,
+    VAR_DNSCRYPT_SECRET_KEY = 466,
+    VAR_DNSCRYPT_PROVIDER_CERT = 467,
+    VAR_DNSCRYPT_PROVIDER_CERT_ROTATED = 468,
+    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE = 469,
+    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS = 470,
+    VAR_DNSCRYPT_NONCE_CACHE_SIZE = 471,
+    VAR_DNSCRYPT_NONCE_CACHE_SLABS = 472,
+    VAR_IPSECMOD_ENABLED = 473,
+    VAR_IPSECMOD_HOOK = 474,
+    VAR_IPSECMOD_IGNORE_BOGUS = 475,
+    VAR_IPSECMOD_MAX_TTL = 476,
+    VAR_IPSECMOD_WHITELIST = 477,
+    VAR_IPSECMOD_STRICT = 478,
+    VAR_CACHEDB = 479,
+    VAR_CACHEDB_BACKEND = 480,
+    VAR_CACHEDB_SECRETSEED = 481,
+    VAR_CACHEDB_REDISHOST = 482,
+    VAR_CACHEDB_REDISPORT = 483,
+    VAR_CACHEDB_REDISTIMEOUT = 484,
+    VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM = 485,
+    VAR_FOR_UPSTREAM = 486,
+    VAR_AUTH_ZONE = 487,
+    VAR_ZONEFILE = 488,
+    VAR_MASTER = 489,
+    VAR_URL = 490,
+    VAR_FOR_DOWNSTREAM = 491,
+    VAR_FALLBACK_ENABLED = 492,
+    VAR_ADDITIONAL_TLS_PORT = 493,
+    VAR_LOW_RTT = 494,
+    VAR_LOW_RTT_PCT = 495,
+    VAR_ALLOW_NOTIFY = 496
   };
 #endif
 /* Tokens.  */
@@ -482,34 +490,42 @@ extern int yydebug;
 #define VAR_USE_SYSTEMD 458
 #define VAR_SHM_ENABLE 459
 #define VAR_SHM_KEY 460
-#define VAR_DNSCRYPT 461
-#define VAR_DNSCRYPT_ENABLE 462
-#define VAR_DNSCRYPT_PORT 463
-#define VAR_DNSCRYPT_PROVIDER 464
-#define VAR_DNSCRYPT_SECRET_KEY 465
-#define VAR_DNSCRYPT_PROVIDER_CERT 466
-#define VAR_DNSCRYPT_PROVIDER_CERT_ROTATED 467
-#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE 468
-#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS 469
-#define VAR_DNSCRYPT_NONCE_CACHE_SIZE 470
-#define VAR_DNSCRYPT_NONCE_CACHE_SLABS 471
-#define VAR_IPSECMOD_ENABLED 472
-#define VAR_IPSECMOD_HOOK 473
-#define VAR_IPSECMOD_IGNORE_BOGUS 474
-#define VAR_IPSECMOD_MAX_TTL 475
-#define VAR_IPSECMOD_WHITELIST 476
-#define VAR_IPSECMOD_STRICT 477
-#define VAR_CACHEDB 478
-#define VAR_CACHEDB_BACKEND 479
-#define VAR_CACHEDB_SECRETSEED 480
-#define VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM 481
-#define VAR_FOR_UPSTREAM 482
-#define VAR_AUTH_ZONE 483
-#define VAR_ZONEFILE 484
-#define VAR_MASTER 485
-#define VAR_URL 486
-#define VAR_FOR_DOWNSTREAM 487
-#define VAR_FALLBACK_ENABLED 488
+#define VAR_ROOT_KEY_SENTINEL 461
+#define VAR_DNSCRYPT 462
+#define VAR_DNSCRYPT_ENABLE 463
+#define VAR_DNSCRYPT_PORT 464
+#define VAR_DNSCRYPT_PROVIDER 465
+#define VAR_DNSCRYPT_SECRET_KEY 466
+#define VAR_DNSCRYPT_PROVIDER_CERT 467
+#define VAR_DNSCRYPT_PROVIDER_CERT_ROTATED 468
+#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE 469
+#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS 470
+#define VAR_DNSCRYPT_NONCE_CACHE_SIZE 471
+#define VAR_DNSCRYPT_NONCE_CACHE_SLABS 472
+#define VAR_IPSECMOD_ENABLED 473
+#define VAR_IPSECMOD_HOOK 474
+#define VAR_IPSECMOD_IGNORE_BOGUS 475
+#define VAR_IPSECMOD_MAX_TTL 476
+#define VAR_IPSECMOD_WHITELIST 477
+#define VAR_IPSECMOD_STRICT 478
+#define VAR_CACHEDB 479
+#define VAR_CACHEDB_BACKEND 480
+#define VAR_CACHEDB_SECRETSEED 481
+#define VAR_CACHEDB_REDISHOST 482
+#define VAR_CACHEDB_REDISPORT 483
+#define VAR_CACHEDB_REDISTIMEOUT 484
+#define VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM 485
+#define VAR_FOR_UPSTREAM 486
+#define VAR_AUTH_ZONE 487
+#define VAR_ZONEFILE 488
+#define VAR_MASTER 489
+#define VAR_URL 490
+#define VAR_FOR_DOWNSTREAM 491
+#define VAR_FALLBACK_ENABLED 492
+#define VAR_ADDITIONAL_TLS_PORT 493
+#define VAR_LOW_RTT 494
+#define VAR_LOW_RTT_PCT 495
+#define VAR_ALLOW_NOTIFY 496
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
@@ -520,7 +536,7 @@ union YYSTYPE
 
 	char*	str;
 
-#line 524 "util/configparser.h" /* yacc.c:1909  */
+#line 540 "util/configparser.h" /* yacc.c:1909  */
 };
 
 typedef union YYSTYPE YYSTYPE;
